//+------------------------------------------------------------------+
//| Last edit: August 8, 2014                    00z Customizer.mq4  |
//| A Chart Window (Helper) Indicator.                               |
//| Manages setting the SMA Period for All Indicators w. prefix 00z. |
//| Manages switching between Dark & Light Background for All.       |
//| User sets SMA from Input box or as Text in the on-screen Label.  |
//| If absent All Indicators revert to programmed defaults:          |
//| Simple MA Period = 20. Background: Dark = 1.                     |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_chart_window

extern string       Manage = "    Settings for All 00z Indicators";
extern string  SMA_Period  = "    Default Simple MA Period = 20";
extern int       smaPeriod = 20;  // 20;
extern string  ScreenColor = "    Dark Screen = 1, Light = 2";
extern int         colorBG = 1;   // 1;
extern string          Set = "    Label Properties";
extern int        fontSize = 24;  // 24; //16..32
extern int          corner = 2;   // 2;
extern string         font = "Arial";      // "Arial"
extern color      labelClr = clrFireBrick; // clrFireBrick;

bool labelsDone = false;
string smaMark = "00z_SMA"; // label: sets SMA period for other 00z prefix indicators
string bgMark  = "00z_BG";  // label: invisible, sets colors according to BG value
//+------------------------------------------------------------------+

int init()
   {
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
   {
   int delLabels = UninitializeReason();
   if(delLabels == REASON_REMOVE || delLabels == REASON_PARAMETERS)
      { // leave screen clean for a new set of Inputs
      ObjectDelete(smaMark);
      ObjectDelete(bgMark);
      }
   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(!labelsDone) // indi does nothing once labels are printed
      {
      //if found label "00z_SMA" read SMA Period from there
      //if not found, SMA Period Input is used
      int getSMA = StrToInteger(ObjectDescription(smaMark));
      if(getSMA > 2) smaPeriod = getSMA;

      ManageLabels();
      }
   return(0);
   } // end int start()

//+------------------------------------------------------------------+

void ManageLabels()
   {
   // All 00z indicators can set from here: SMA Period & Background
   int x = 20; int y = 20;

   string text = DoubleToStr(smaPeriod, 0);
   string obj = smaMark;
   if(!ObjectCreate(obj, OBJ_LABEL,0,0,0))     ObjectCreate(obj, OBJ_LABEL,0,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))   ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))   ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, corner)) ObjectSet(obj, OBJPROP_CORNER, corner);
   if(!ObjectSetText(obj, text, fontSize, font, labelClr)) ObjectSetText(obj, text, fontSize, font, labelClr);

   text = DoubleToStr(colorBG, 0);
   obj = bgMark; // invisible
   if(!ObjectCreate(obj, OBJ_LABEL,0,0,0))     ObjectCreate(obj, OBJ_LABEL,0,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, 10))  ObjectSet(obj, OBJPROP_XDISTANCE, 10);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, 10))  ObjectSet(obj, OBJPROP_YDISTANCE, 10);
      if(!ObjectSet(obj, OBJPROP_CORNER, 2))      ObjectSet(obj, OBJPROP_CORNER, 2);
      if(!ObjectSet(obj, OBJPROP_TIMEFRAMES, -1)) ObjectSet(obj, OBJPROP_TIMEFRAMES, -1);
   if(!ObjectSetText(obj, text, fontSize, font, CLR_NONE)) ObjectSetText(obj, text, fontSize, font, CLR_NONE);

   labelsDone = true;
   return;
   } // end void ManageLabels()

//+--------------------------------EOP-------------------------------+
