//+------------------------------------------------------------------+
//| Last edit: August 8, 2014                   00z Flat Candy.mq4   |
//| Track Z-scores on Close, Bar High & Low, in a separate window.   |
//| SMA = always 0.0! Levels are static StdDev values.               |
//| If default filter (= PI rounded) is set to "false", the user can |
//| set a custom StdDev level line.                                  |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window

#property indicator_buffers 8
#property indicator_color1 clrRed
#property indicator_color2 clrLime
#property indicator_color3 clrDeepSkyBlue
#property indicator_color4 clrGold
#property indicator_color5 clrDeepSkyBlue
#property indicator_color6 clrDeepSkyBlue
#property indicator_color7 clrSilver
#property indicator_color8 clrSilver

extern string      Make_Sure = "    Set SMA Period equal to BB system!";
extern string     SMA_Period = "    Default Simple MA Period = 20";
extern int         smaPeriod = 20; // 20
extern string current_Filter = "    Default PI = 3.141592";
extern bool        Filter_PI = true; // if Filter = "true": use Pi rounded as outside filter
extern double  elseUseCustom = 1.0;  // if Filter = "false": set another positive double here
extern string    ScreenColor = "    Dark Screen = 1, Light = 2";
extern int           colorBG = 1; // 1

// Indicator Buffers
double zHigh[];     // bar high z-score
double zLow[];      // bar low z-score
double zScore[];    // z-score on Close
double SMAzero[];   // flat level lines
double filterHi[];  // user's choice
double filterLo[];
double stdDevHi2[]; // fixed at 2.0 SD
double stdDevLo2[];

double PI = 3.141592; //not mathPI
int limit; // bars
int winID; // subwindow
color clrHi, clrLo, clrZ, clrText, clrLvl, clrSMA;

//+------------------------------------------------------------------+

void SetColors(int backGround)
   { // do not break lines
   switch (backGround)
      {
      case 1: clrHi= clrRed; clrLo = clrLime; clrZ = clrDeepSkyBlue;
              clrSMA = clrGold; clrText = clrGold; clrLvl = clrSilver; return;

      case 2: clrHi= clrFireBrick; clrLo = clrGreen; clrZ = clrBlue;
              clrSMA = Black; clrText = clrBlack; clrLvl = clrBlack; return;

      default:     return;
      }
   } // end void SetColors(int backGround)

//+------------------------------------------------------------------+

int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;
   limit = Bars - smaPeriod - 1;

   //if found label "00z_BG" read BG selection from there
   int getBG = StrToInteger(ObjectDescription("00z_BG"));
   if(getBG > 0) colorBG = getBG;
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   SetColors(colorBG);
   int width = colorBG; // set some plot lines widht at 2 for Light BG

   if(Filter_PI) elseUseCustom = PI; // Pi rounded
   elseUseCustom = MathAbs(elseUseCustom);

   SetIndexBuffer(0,zHigh);
   SetIndexStyle (0,DRAW_LINE,STYLE_SOLID, width, clrHi);
   SetIndexLabel (0,"zHigh");

   SetIndexBuffer(1,zLow);
   SetIndexStyle (1,DRAW_LINE,STYLE_SOLID, width, clrLo);
   SetIndexLabel (1,"zLow");

   SetIndexBuffer(2,zScore);
   SetIndexStyle (2,DRAW_LINE,STYLE_DOT,1, clrZ);
   SetIndexLabel (2,"zScore");

   SetIndexBuffer(3,SMAzero);
   SetIndexStyle (3,DRAW_LINE,STYLE_SOLID, width, clrSMA);
   SetIndexLabel (3,NULL);

   SetIndexBuffer(4,filterHi);
   SetIndexStyle (4,DRAW_LINE,STYLE_DASHDOT,1, clrZ);
   SetIndexLabel (4,"Filter");

   SetIndexBuffer(5,filterLo);
   SetIndexStyle (5,DRAW_LINE,STYLE_DASHDOT,1, clrZ);
   SetIndexLabel (5,NULL);

   SetIndexBuffer(6,stdDevHi2);
   SetIndexStyle (6,DRAW_LINE,STYLE_DASHDOTDOT,1, clrLvl);
   SetIndexLabel (6,"stdDev2");

   SetIndexBuffer(7,stdDevLo2);
   SetIndexStyle (7,DRAW_LINE,STYLE_DASHDOTDOT,1, clrLvl);
   SetIndexLabel (7,NULL);

   IndicatorShortName("Z_Flat_Candy");

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
   {
   ObjectDelete("sma_period");
   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(winID < 1)
      {
      winID = WindowFind("Z_Flat_Candy");
      // setting label to show current SMA period
      string obj = "sma_period";
      string text = StringConcatenate(Symbol(), "  ", smaPeriod, " SMA");
      ObjectCreate(obj, OBJ_LABEL,winID,0,0);
         ObjectSet(obj, OBJPROP_XDISTANCE,10);
         ObjectSet(obj, OBJPROP_YDISTANCE,20);
         ObjectSet(obj, OBJPROP_CORNER,0);
      ObjectSetText(obj, text, 9, "Arial", clrText);
      }

   int i,Counted_bars;
   Counted_bars=IndicatorCounted();
   i=Bars-Counted_bars-1;
   if (i>limit-1)i=limit-1;

   while(i >= 0)
      {
      double SMA =     iMA(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, i);
      double SD  = iStdDev(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, i);

      zHigh[i]  =  (High[i] - SMA) / SD;
      zLow[i]   =   (Low[i] - SMA) / SD;
      zScore[i] = (Close[i] - SMA) / SD;
      // setting static level values
      SMAzero[i]   =   0.0; //  don't modify, baseline value
      stdDevHi2[i] =   2.0; //  2.0;
      stdDevLo2[i] = - 2.0; // -2.0;
      filterHi[i]  =   elseUseCustom;
      filterLo[i]  = - elseUseCustom;

      i--;
      } // end while(i >= 0)

   return(0);
   } // end int start()

//+------------------------------EOP---------------------------------+
