//+------------------------------------------------------------------+
//|   Last edit: August 8, 2014                   00z Multi TF.mq4   |
//|   On the Left:  Show z-scores on all 9 TFs + minimax values.     |
//|   On the Right: Specific values for the Current TF only.         |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_chart_window

extern string  Make_Sure = "    Set SMA Period equal to BB system!";
extern string SMA_Period = "    Default Simple MA Period = 20";
extern int     smaPeriod = 20;  // 20
extern color     clrText = clrGray;      // default only, user color choice
extern color     clrMaxZ = clrLimeGreen; // clrLimeGreen; // -Point out TFs with the
extern color     clrMinZ = clrOrangeRed; // clrOrangeRed; // -Min and Max z-scores
extern string   textFont = "Arial";
extern int      fontSize = 9;   // 9, Any
extern int        Corner = 0;  // 0 .. 3
extern int    moveLabels = 30; // vertical offset from top or bottom of screen

string tf[9] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int   per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
      PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

// Dev refers to the Price Deviation or Price Difference Score
string rowText[6] = {"", "Bid", "SMA", "Dev", "SD", "Z"};
string data[6];     // current TF info
double tfZ[9];      // all TF z-scores
double zMax, zMin;  // minimax z-high and low
string pref = "z_"; // delete graphics
double xPoint;      // display pips
color clrHold;

//+------------------------------------------------------------------+

int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;
   clrHold = clrText;

   xPoint = Point;
   if(Digits==5||Digits==3) {xPoint = Point * 10;}
   if(Digits==2) {xPoint = Point * 100;}

   for (int i=0; i<=8 ;i++) {if(per[i] == Period()) rowText[0] = tf[i];}

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
  {
   Clear(pref, 0);
   return(0);
  } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   GetData();
   ShowLabels();

return(0);
   } // end int start()

//+------------------------------------------------------------------+

void GetData()
   {
   // get data for current TF
   double SMA =     iMA(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE,0);
   double SD  = iStdDev(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE,0);
   double Dev = (Bid - SMA) / xPoint; // display Price Difference as pips
   double Z   = (Bid - SMA) / SD;

   // load z-score array for all 9 TFs
   for (int i=0; i<=8 ;i++) tfZ[i] = GetZ(per[i], 0);

   // get minimax z
   zMax = tfZ[ArrayMaximum(tfZ)];
   zMin = tfZ[ArrayMinimum(tfZ)];

   // load labels for current TF
   data[0] = StringConcatenate(smaPeriod, " sma");
   data[1] = DoubleToStr(Bid, Digits - 1);
   data[2] = DoubleToStr(SMA, Digits - 1);
   data[3] = StringConcatenate(GetSign(Dev), DoubleToStr(Dev, 1));
   data[4] = DoubleToStr(SD / xPoint, 1);
   data[5] = StringConcatenate(GetSign(Z), DoubleToStr(Z, 2));

   return;
   } // end void GetData()

//+------------------------------------------------------------------+

void ShowLabels()
   {// four columns
   int x = 22; int y = 1.8 * fontSize;

   for(int i=0; i<=8; i++)
      {
      string obj = StringConcatenate("a_", i);
      Label(obj, x, i*y + moveLabels, Corner, tf[i], fontSize, textFont, clrText);
      }

   x += 4 * fontSize;
   for(i=0; i<=8; i++)
      {
      if(tfZ[i] == zMax) clrText = clrMaxZ;
      if(tfZ[i] == zMin) clrText = clrMinZ;
      string text = StringConcatenate(GetSign(tfZ[i]), DoubleToStr(tfZ[i], 2));
      obj = StringConcatenate("b_", i);
      Label(obj, x, i*y + moveLabels, Corner, text, fontSize, textFont, clrText);
      clrText = clrHold;
      }

   x += 6 * fontSize;
   for(i=0; i<=5; i++)
      {
      obj = StringConcatenate("c_", i);
      Label(obj, x, i*y + moveLabels, Corner, rowText[i], fontSize, textFont, clrText);
      }

   x += 4 * fontSize;
   for(i=0; i<=5; i++)
      {
      obj = StringConcatenate("d_", i);
      Label(obj, x, i*y + moveLabels, Corner, data[i], fontSize, textFont, clrText);
      }

   return;
   } // end void ShowLabels()

//+------------------------------SUPPORT-----------------------------+

void Label(string obj, int x, int y, int corner, string text, int size, string font, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_LABEL, 0,0,0))    ObjectCreate(obj, OBJ_LABEL, 0,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))   ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))   ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, corner)) ObjectSet(obj, OBJPROP_CORNER, corner);
   if(!ObjectSetText(obj, text, size, font, clr)) ObjectSetText(obj, text, size, font, clr);

   return;
   }// end void Label(

//+------------------------------------------------------------------+
string GetSign(double in)
   {
   if (in <= 0) {string sign = "";} else {sign = "+";}
   return(sign);
   } // end string GetSign(double in)

//+------------------------------------------------------------------+

double GetZ(int period, int bar)
   { // Bar Close z-score
   return((iClose(NULL, period, bar) -
              iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))/
          iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetZ(int per, int bar)

//+------------------------------------------------------------------+
void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+--------------------------------EOP-------------------------------+
