//+------------------------------------------------------------------+
//|  Last edit: August 8, 2014                      00z PumpJack.mq4 |
//|  REAL TIME only. Values are Z-scores on CLOSE on each TF #0 Bar. |
//|  Shown in a dynamic display modelled as a "nodding donkey".      |
//|  Also shows the average or Mean Z-score for all 9 TFs.           |
//+------------------------------------------------------------------+
#property copyright " 2012-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window

#property indicator_buffers 4
#property indicator_color1 clrBrown
#property indicator_color2 clrOrange
#property indicator_color3 clrLightSeaGreen
#property indicator_color4 clrLightSeaGreen

extern string    Make_Sure = "    Set SMA Period equal to BB system!";
extern string   SMA_Period = "    Default Simple MA Period = 20";
extern int       smaPeriod = 20; // 20;
extern int          anchor = 0;  // 0;  //Moves indicator to the left
extern int            step = 4;  // 4;  //Use higher to stretch indi when zoomed out
extern string  ScreenColor = "    Dark Screen = 1, Light = 2";
extern int         colorBG = 1;  // 1;
extern string         font = "Arial";
extern int       scoreSize = 11; // 11; //Fontsize, only for z-scores display
extern int        infoSize = 9;  // 9;  //Information fontsize

string tf[10] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN","mean Z"};
int    per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
       PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

// 2 z-score indicator buffers
double tfZ[];   // plot changing values of 9 individual TF z-scores
double zMean[]; // plot the average or Mean Z-score value

// 2 StdDev level lines set at 2.0 SD
double hiLevel[];
double loLevel[];

int winID; //subwindow
string pref  = "pump_"; // delete graphics

color clrTfZ, clrZMean, clrInfo;  // clr indi buffers 0 and 1, infotext
color clrLvl_0, clrLvl_2;         // clr level lines 0 and 2 StdDev
color z_1, z1_2, z2_3, z3_4, z_4; // clr z-scores, 1 per unit of SD
color clrMeanline;                // clr meanZ line

//+------------------------------------------------------------------+

void SetColors(int backGround)
   { // modify as wanted but don't break lines.
   switch (backGround)
      {
      case 1: clrTfZ= clrBrown; clrZMean= clrOrange; clrInfo= clrGold;
              clrLvl_0= clrLightGoldenrod; clrLvl_2= clrLightSeaGreen;
              z_1= clrWhite; z1_2= clrHotPink; z2_3= clrLime; z3_4= clrYellow; z_4= clrRed;
              clrMeanline= clrGold;
              return;

      case 2: clrTfZ= clrFireBrick; clrZMean= clrDarkOrchid; clrInfo= clrBlack;
              clrLvl_0= clrBlack; clrLvl_2= clrBlack;
              z_1= clrBlack; z1_2= clrBlue; z2_3= clrGreen; z3_4= clrDarkOrange; z_4= clrRed;
              clrMeanline= clrBlack;
              return;

      default:     return;
      }
   } // end void SetColors(int backGround)

//+------------------------------------------------------------------+
int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;

   //if found label "00z_BG" read BG selection from there
   int getBG = StrToInteger(ObjectDescription("00z_BG"));
   if(getBG > 0) colorBG = getBG;
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   SetColors(colorBG);
   int width = colorBG; // set zMean plot line widht at 2 for Light BG

   SetIndexBuffer(0,tfZ);
    SetIndexStyle(0,DRAW_SECTION,STYLE_DOT, 1, clrTfZ);
    SetIndexLabel(0,"TF Z Scores");

   SetIndexBuffer(1,zMean);
    SetIndexStyle(1,DRAW_LINE,STYLE_SOLID, width, clrZMean);
    SetIndexLabel(1,"zMean");

   SetIndexBuffer(2,hiLevel);
    SetIndexStyle(2,DRAW_LINE,STYLE_DASHDOT,1, clrLvl_2);
    SetIndexLabel(2,NULL);

   SetIndexBuffer(3,loLevel);
    SetIndexStyle(3,DRAW_LINE,STYLE_DASHDOT,1, clrLvl_2);
   SetIndexLabel (3,NULL);

   SetLevelStyle(STYLE_DOT, 1, clrLvl_0);
   SetLevelValue(0, 0);
   //SetLevelValue(1, 1.00);
   //SetLevelValue(2, -1.00);

   IndicatorShortName("pumpJack");

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
   {
   Clear(pref, 0);
   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(winID < 1) winID = WindowFind("pumpJack");

   for(int i=0;i<=step*18+anchor;i++)
      {
      hiLevel[i]= 2.0; loLevel[i]= -2.0;
      tfZ[i] = EMPTY_VALUE;
      }

   double sumZ = 0, sumAbsZ = 0, posLoad = 0, negLoad = 0;
   for(i=0; i<=8 ;i++)
      {
      tfZ[step*i+anchor] = GetZ(per[i], 0);   // get individual TF z-scores
      ShowZ(i, tfZ[step*i+anchor]);           // show individual TF z-scores
      sumZ += tfZ[step*i+anchor];             // sum all scores
      sumAbsZ += MathAbs(tfZ[step*i+anchor]); // sum all MathAbs(scores)
      if(tfZ[step*i+anchor] >= 0) posLoad += tfZ[step*i+anchor]; // sum positive scores
      if(tfZ[step*i+anchor] <  0) negLoad += tfZ[step*i+anchor]; // sum negative scores
      }

   double meanZ = sumZ / 9;          // average z-score
   double averageAbsZ = sumAbsZ / 9; // average Absolute z-score
   ShowZ(9, meanZ);       // display Mean Z value as text
   zMean[anchor] = meanZ; // plot the Mean Z buffer, slow on higher TF

   // info labels on the left side of display
   Labels(Symbol());      Labels(StringConcatenate(smaPeriod, " SMA"));
   Labels("Mean Z");      Labels(StringConcatenate(GetSign(meanZ),   DoubleToStr(meanZ, 4)));
   Labels(" ");           Labels(" ");
   Labels("Positive Z");  Labels(StringConcatenate(GetSign(posLoad), DoubleToStr(posLoad, 4)));
   Labels("Negative Z");  Labels(StringConcatenate(GetSign(negLoad), DoubleToStr(negLoad, 4)));
   Labels("Sum Z");       Labels(StringConcatenate(GetSign(sumZ),    DoubleToStr(sumZ, 4)));
   Labels(" ");           Labels(" ");
   Labels("experiment:"); Labels("ratio z / abs(z)");
   Labels("sum abs(z)");  Labels(DoubleToStr(sumAbsZ, 4));
   Labels("mean abs(z)"); Labels(DoubleToStr(averageAbsZ, 4));
   Labels("% z/abs(z)");  Labels(StringConcatenate(DoubleToStr((sumZ / sumAbsZ) * 100, 2), "%" ));
   // That last, a fast percentage coefficient of a skewness? over the 9 TFs?
   // At +/-100%, price is on the same side of all TF SMAs! Follow-up.

   // horizontal mean Z line
   TrendLine("zMeanLine", step*8+anchor, anchor, meanZ, meanZ, clrMeanline, 2, false, true);

   return(0);
   } // end int start()

//+------------------------------------------------------------------+

void ShowZ(int i, double zScore)
   {
   // show z-scores and TF labels
   // set distinctive z-score colors
   double z = MathAbs(zScore); //
   if (z<1)   color clr = z_1;
   if (z>=1 && z<2) clr = z1_2;
   if (z>=2 && z<3) clr = z2_3;
   if (z>=3 && z<4) clr = z3_4;
   if (z>=4)        clr = z_4;

   // show on-tick values of TF z-scores at SD scale
   string text = StringConcatenate(GetSign(zScore), DoubleToStr(zScore, 2));
   string obj  = StringConcatenate(tf[i], "_z");
   TextBox(obj, step*i+anchor, zScore, text, scoreSize, font, clr);

   // set up TF text boxes on 0 level
   text = tf[i];
   obj  = StringConcatenate(tf[i], "_txt");
   TextBox(obj, step*i+anchor, 0, text, scoreSize-4, font, clr);

   return;
   } // end void ShowZ(int i, double zScore)

//+------------------------------------------------------------------+

void Labels(string data)
   {// show information on the left of display
   static int d = 0; // label ID
   static int x = 22;
   static int y = 22;

   string obj = StringConcatenate("info_", d);
   Label(obj, x, y, 0, data, infoSize, font, clrInfo);

   d++;
   x += 80;

   if(d % 2 == 0) {x= 22; y += infoSize * 2;}
   if(d % 22 == 0) {d = 0; x = 22; y = 22;}

   return;
   } // end void Labels(string data)

//+-----------------------------SUPPORT------------------------------+

void TrendLine(string obj, int t1, int t2, double p1, double p2, color clr, int style, bool ray, bool back)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_TREND, winID,0,0,0,0)) ObjectCreate(obj, OBJ_TREND, winID,0,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1]))     ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_TIME2, Time[t2]))     ObjectSet(obj, OBJPROP_TIME2, Time[t2]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))          ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_PRICE2, p2))          ObjectSet(obj, OBJPROP_PRICE2, p2);
      if(!ObjectSet(obj, OBJPROP_COLOR,  clr))         ObjectSet(obj, OBJPROP_COLOR,  clr);
      if(!ObjectSet(obj, OBJPROP_STYLE,  style))       ObjectSet(obj, OBJPROP_STYLE,  style);
      if(!ObjectSet(obj, OBJPROP_RAY,    ray))         ObjectSet(obj, OBJPROP_RAY,    ray);
      if(!ObjectSet(obj, OBJPROP_BACK,   back))        ObjectSet(obj, OBJPROP_BACK,   back);

   return;
   }// end void TrendLine(

//+------------------------------------------------------------------+

void TextBox(string obj, int t1, double p1, string text, int size, string font_, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if (!ObjectCreate(obj, OBJ_TEXT, winID,0,0,0)) ObjectCreate(obj, OBJ_TEXT, winID,0,0,0);
       if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1]))  ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
       if(!ObjectSet(obj, OBJPROP_PRICE1, p1))       ObjectSet(obj, OBJPROP_PRICE1, p1);
   if(!ObjectSetText(obj, text, size, font_, clr)) ObjectSetText(obj, text, size, font_, clr);

   return;
   }// end void TextBox(

//+------------------------------------------------------------------+

void Label(string obj, int x, int y, int corner, string text, int size, string font_, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_LABEL, winID,0,0)) ObjectCreate(obj, OBJ_LABEL, winID,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))    ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))    ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, corner))  ObjectSet(obj, OBJPROP_CORNER, corner);
   if(!ObjectSetText(obj, text, size, font_, clr)) ObjectSetText(obj, text, size, font_, clr);

   return;
   }// end void Label(

//+------------------------------------------------------------------+

string GetSign(double in)
   {
   if (in <= 0) {string sign = " ";} else {sign = "+";}
   return(sign);
   } // end string GetSign(double in)

//+------------------------------------------------------------------+

double GetZ(int period, int bar)
   { // Bar Close z-score
   return((iClose(NULL, period, bar) -
              iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))/
          iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetZ(int period, int bar)

//+------------------------------------------------------------------+

void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+-----------------------------EOP----------------------------------+