//+------------------------------------------------------------------+
//| Last edit: August 8, 2014           00z Write MTF Historical.mq4 |
//| Write historical Z-scores on Close, Bar High & Low.              |
//| Each selected TF is written sequentially to a single file.       |
//| Deactivate or remove indicator to unlock that data file.         |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window

#property indicator_buffers 3
#property indicator_color1 clrLightGray
#property indicator_color2 clrDarkSlateGray
#property indicator_color3 clrGray
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 1
#property indicator_style3 2

extern string By_Default = "    History Writer is Inactive.";
extern bool  Activate_On = false; // use as On/Off toggle
extern string  Make_Sure = "    Set SMA Period equal to BB system!";
extern string SMA_Period = "    Default Simple MA Period = 20";
extern int     smaPeriod = 20;  // 20;
extern int       maxBars = 1000;  // 1000; // free user choice
extern color    clrLabel = clrDodgerBlue;  //clrDodgerBlue;
extern string       font = "Arial"; // label font

// Indicator Buffers
double zHigh[];
double zLow[];
double zScore[];

// Dev refers to the Price Deviation or Price Difference Score
double tStamp, SMA, SD, Dev, barHigh, barLow;
string tF;

string fileName;
int    handle, winID, limit;
int    rec_ID; // record ID for each data line (duplicates!)
int    tf_ID;  // unique record ID for each TF
double xPoint; // display pips
bool   done;   // work done

//+------------------------------------------------------------------+

int init()
   {
   if(Activate_On)
      {
      //if found label "00z_SMA" read SMA Period from there
      int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
      if(getSMA > 2) smaPeriod = getSMA;

      limit = Bars - smaPeriod - 1;
      if(maxBars > limit) maxBars = limit;

      xPoint = Point;
      if(Digits==5||Digits==3) {xPoint = Point * 10;}
      if(Digits==2) {xPoint = Point * 100;}

      SetIndexBuffer(0,zHigh);
      SetIndexStyle (0,DRAW_LINE,STYLE_SOLID);
      SetIndexLabel (0,"zHigh");

      SetIndexBuffer(1,zLow);
      SetIndexStyle (1,DRAW_LINE,STYLE_SOLID);
      SetIndexLabel (1,"zLow");

      SetIndexBuffer(2,zScore);
      SetIndexStyle (2,DRAW_LINE);
      SetIndexLabel (2,"zScore");

      SetLevelStyle(STYLE_DOT,0, Gray);
      SetLevelValue(0,   0.0);
      SetLevelValue(1,   1.0);
      SetLevelValue(2, - 1.0);
      SetLevelValue(3,   2.0);
      SetLevelValue(4, - 2.0);
      SetLevelValue(5,   3.0);
      SetLevelValue(6, - 3.0);

      TF(Period());
      // write header. file stays open until indi removal or user deactivation
      fileName = StringConcatenate("zKit\\MTF Hist\\",
                                   Symbol(), "_sma", smaPeriod, ".txt");

      handle = FileOpen(fileName, FILE_CSV|FILE_READ|FILE_WRITE, '\t');
      // FileWrite(handle, fileName);
      FileWrite(handle, "Rec#", "Date", "TF", "Price", "SMA",
                        "Deviation", "SD Pips", "SD", "Z-Score",
                        "Bar High", "Z-High", "Bar Low", "Z-Low",
                        "TF Rank", "SMA Per", "", fileName);
      FileSeek(handle, 0, SEEK_END);

      done = false;
      } // end if(Activate_On)

   IndicatorShortName("Z MTF Historical");
   return(0);
   }

//+------------------------------------------------------------------+

int deinit()
   {
   ObjectDelete("z_historical");
   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(winID < 1) winID = WindowFind("Z MTF Historical");
   if(!Activate_On) {Label("Z History Writer is Inactive."); return(0);}

   string text = StringConcatenate(smaPeriod, " SMA ", Symbol(), ", ", tF,
                                   "  Ready:  Bars Written = ", rec_ID);
   Label(text);

   while(!done)
      {
      int i = maxBars - 1; // total count includes a Bar #0

      while(i >= 0)
         {
         rec_ID++;
         tStamp  = Time[i];
         barHigh = High[i];
         barLow  =  Low[i];

         SMA =     iMA(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, i);
         SD  = iStdDev(NULL,0, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, i);
         Dev = Close[i] - SMA; // price deviation

         zHigh[i]  =  (High[i] - SMA) / SD;
         zLow[i]   =   (Low[i] - SMA) / SD;
         zScore[i] = (Close[i] - SMA) / SD;

         FileWrite(handle, i, TimeToStr(tStamp), tF, Close[i], SMA,
                   NormalizeDouble((Dev / xPoint), 1),
                   NormalizeDouble((SD / xPoint), 1), SD, zScore[i],
                   barHigh, zHigh[i], barLow, zLow[i], tf_ID, smaPeriod);

         text = StringConcatenate(smaPeriod, " SMA ", Symbol(), " ", tF,
                                  ". Bars = ", rec_ID, "  BUSY Writing ...");
         Label(text);
         i--;
         } // end while(i >= 0)

      FileFlush(handle);
      done = true;
      } // end while(!done)

   return(0);
   } // end int start()

//+------------------------------------------------------------------+

void Label(string text)
   {
   // setting information label
   string obj = "z_historical";
   if(!ObjectCreate(obj, OBJ_LABEL,winID,0,0)) ObjectCreate(obj, OBJ_LABEL,winID,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE,10))   ObjectSet(obj, OBJPROP_XDISTANCE,10);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE,20))   ObjectSet(obj, OBJPROP_YDISTANCE,20);
      if(!ObjectSet(obj, OBJPROP_CORNER,0))       ObjectSet(obj, OBJPROP_CORNER,0);

   if (!ObjectSetText(obj, text, 10, font, clrLabel)) ObjectSetText(obj, text, 10, font, clrLabel);

   return;
   } // end void ManageLabel()

//+---------------------------SUPPORT--------------------------------+

void TF(int period)
   {
   switch (period)
      {
      case PERIOD_M1:  tf_ID = 0; tF ="M1";  return;
      case PERIOD_M5:  tf_ID = 1; tF ="M5";  return;
      case PERIOD_M15: tf_ID = 2; tF ="M15"; return;
      case PERIOD_M30: tf_ID = 3; tF ="M30"; return;
      case PERIOD_H1:  tf_ID = 4; tF ="H1";  return;
      case PERIOD_H4:  tf_ID = 5; tF ="H4";  return;
      case PERIOD_D1:  tf_ID = 6; tF ="D1";  return;
      case PERIOD_W1:  tf_ID = 7; tF ="W1";  return;
      case PERIOD_MN1: tf_ID = 8; tF ="MN";  return;

      default:     return;
      }
   } // end void TF(int period)

//+------------------------------EOP---------------------------------+