//+------------------------------------------------------------------+
//|  Last edit: August 8, 2014           00z Write MTF Real Time.mq4 |
//|  Real Time. Write complete TF sets to CSV file.                  |
//|  Use w. allTicks setting or on M1 TF to fully build all 9 TFs.   |
//|  Otherwise Lower TFs are missed, as fewer updates will occur.    |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_chart_window

extern string By_Default = "    Real Time Writer is Inactive.";
extern bool  Activate_On = false; // use as On/Off toggle
extern string  Make_Sure = "    Set SMA Period equal to BB system!";
extern string SMA_Period = "    Default Simple MA Period = 20";
extern int     smaPeriod = 20;    // 20;
extern string  Bars_Only = "    By Default values are written on Bar Close Only.";
extern bool     allTicks = false; // if "true", write values on every tick
extern int       maxSets = 1000;  // 1000; sets = 9000 records
extern string     Manage = "    Label";
extern color   labelClr  = clrDodgerBlue; //clrDodgerBlue;
extern string       font = "Arial";   // label font
extern int      fontSize = 10; // 10; // label fontsize
extern int        corner = 0;  // 0;  // label display
extern int             x = 22; // 22;
extern int             y = 22; // 22;


string tf[9] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int   per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
      PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

// Dev[] refers to the Price Deviation or Price Difference Score
double tStamp[9], SMA[9], SD[9], Dev[9], tfZ[9],
       barHigh[9], zHigh[9], barLow[9], zLow[9];

string fileName, thisTF;
bool   newBar;
int    numBars, handle, ticks; // note: first Bar tick count is incomplete!
int    set_ID; // unique record ID for each set of 9 TFs
double xPoint; // display pips
bool   done;   // flag work done
//string status = "  Running";

//+------------------------------------------------------------------+

int init()
   {
   if(Activate_On)
      {
      //if found label "00z_SMA" read SMA Period from there
      int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
      if(getSMA > 2) smaPeriod = getSMA;

      xPoint = Point;
      if(Digits==5||Digits==3) {xPoint = Point * 10;}
      if(Digits==2) {xPoint = Point * 100;}
      for (int i=0; i<=8 ;i++) {if(per[i] == Period()) thisTF = tf[i];}
      numBars = Bars;
      newBar = false;
      done = false;

      // write header. file stays open until next deinit() or user close
      fileName = StringConcatenate("zKit\\MTF Real\\",
                                   Symbol(), "_", Period(), "_sma", smaPeriod,
                                   "_", TimeCurrent(), ".txt");

      handle = FileOpen(fileName, FILE_CSV|FILE_WRITE, '\t');
      // FileWrite(handle,fileName); // moved to end of header = spreadsheet sorting
      FileWrite(handle, "Set_ID", "Date", "TF", "Price", "SMA",
                        "Deviation", " SD Pips", "SD",
                        "Z-Score", "Bar High", "Z-High", "Bar Low", "Z-Low",
                        "TF Rank", "SMA Per", "Ticks", fileName);
      } // end if(Activate_On)

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

void WriteToFile()
   { // moved here for easier modding
   set_ID++;
   for ( int i=0; i<=8 ;i++ )
      {
      FileWrite(handle, set_ID, TimeToStr(tStamp[i]), tf[i], Bid, SMA[i],
               NormalizeDouble((Dev[i] / xPoint), 1),
               NormalizeDouble((SD[i] / xPoint), 1), SD[i],
               tfZ[i], barHigh[i], zHigh[i], barLow[i], zLow[i],
               i, smaPeriod, ticks);
      }
   newBar = false;

   return;
   } // end void WriteToFile()

//+------------------------------------------------------------------+

int deinit()
   {
   FileClose(handle);
   ObjectDelete("z_realtime");

   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(!done)
      {
      if(!Activate_On) {Label("Z Real Time Writer is Inactive."); return(0);}

      string text = StringConcatenate(smaPeriod, " SMA ", Symbol(), ", ", thisTF,
                      "  RT running: TF Sets Written = ", set_ID, "  Tick Count = ", ticks);
      Label(text);
      ticks++;

      // on New Bar Open: write all values for prior Bar
      if(numBars < Bars) {numBars = Bars; newBar = true;}
      if(newBar && !allTicks) WriteToFile();

      // get data for all 9 TFs
      for(int i=0; i<=8 ;i++)
         {
         tStamp[i]  = iTime(NULL, per[i], 0);
         barHigh[i] = iHigh(NULL, per[i], 0);
         barLow[i]  =  iLow(NULL, per[i], 0);

         SMA[i] =     iMA(NULL, per[i], smaPeriod, 0,MODE_SMA,PRICE_CLOSE,0);
         SD[i]  = iStdDev(NULL, per[i], smaPeriod, 0,MODE_SMA,PRICE_CLOSE,0);

         Dev[i]   = Bid - SMA[i]; // price deviation
         tfZ[i]   = Dev[i] / SD[i];
         zHigh[i] = (barHigh[i] - SMA[i]) / SD[i];
         zLow[i]  =  (barLow[i] - SMA[i]) / SD[i];
         } // end for(int

      if(allTicks) WriteToFile(); // write all values every tick
      } //end if(!done)

   return(0);
   } // end int start()

//+------------------------------------------------------------------+

void Label(string text)
   {
   //setting information label
   string obj  = "z_realtime";
   if(!ObjectCreate(obj, OBJ_LABEL,0,0,0))     ObjectCreate(obj, OBJ_LABEL,0,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))   ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))   ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, corner)) ObjectSet(obj, OBJPROP_CORNER, corner);
   if(!ObjectSetText(obj, text, fontSize, font, labelClr)) ObjectSetText(obj, text, fontSize, font, labelClr);

   return;
   } // end void Label()

//+--------------------------------EOP-------------------------------+
