//+------------------------------------------------------------------+
//|   DEMO                             00z 3 Basic Z Functions.mq4   |
//|   Last edit: August 8, 2014                                      |
//|   3 Functions that return() Bar Close, High and Low Z-SCORES;    |
//|   double GetZ(int period, int bar), GetZHi(.,.), GetZLo(.,.)     |
//|   Each function can reference any TF and Bar, by passing a       |
//|   specific Period() and Bar Index.                               |
//|                                                                  |
//|   In this DEMO we ONLY call the current Bars #0 on Every Tick.   |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window

extern string  Make_Sure = "    Set SMA Period equal to BB system!";
extern string SMA_Period = "    Default Simple MA Period = 20";
extern int     smaPeriod = 20; // 20
extern string       font = "Arial";
extern string   fontSize = 10; // 10
extern color     clrText = clrGray; // default only, user color choice

string  tf[9] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
       PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

string pref = "z3x_";
int winID; // subwindow

//+------------------------------------------------------------------+

int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;

   IndicatorShortName("3_Z_Functions");
   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
  {
   Clear(pref, 0);
   return(0);
  } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(winID < 1) winID = WindowFind("3_Z_Functions");

   Labels(StringConcatenate(smaPeriod, " SMA"));
   for(int i=8; i>=0; i--) Labels(tf[i]); // set the TF header line

   Labels("Z High");   // Bar High z-scores
   for(i=8; i>=0; i--) Labels(DoubleToStr(GetZHi(per[i], 0), 8));

   Labels("Z Score");  // Bar Close z-scores
   for(i=8; i>=0; i--) Labels(DoubleToStr(GetZ(per[i], 0), 8));

   Labels("Z Low");    // Bar Low z-scores
   for(i=8; i>=0; i--) Labels(DoubleToStr(GetZLo(per[i], 0), 8));

   return(0);
   } // end int start()

//+---------------------------FUNCTIONS------------------------------+
/*
 z-score = (observed value - mean) / standard deviation
 z-score = (Bid - SMA) / SD

 call: GetZ(Period(), BarIndex); ==> GetZ(0, 0); calls current Period, Bar 0
*/
//+------------------------------------------------------------------+

double GetZ(int period, int bar)
   { // Bar Close z-score
   return((iClose(NULL, period, bar) -
              iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))/
          iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetZ(int period, int bar)

//+------------------------------------------------------------------+

double GetZHi(int period, int bar)
   { // Bar High z-score
   return((iHigh(NULL, period, bar) -
             iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))/
         iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetZHi(int period, int bar)

//+------------------------------------------------------------------+

double GetZLo(int period, int bar)
   { // Bar Low z-score
   return((iLow(NULL, period, bar) -
            iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))/
        iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetZLo(int period, int bar)

//+---------------------------END FUNCTIONS--------------------------+

void Labels(string data)
   {
   static int d = 0; // label ID
   static int x = 22;
   static int y = 22;

   string obj = StringConcatenate(pref, d);
   if(!ObjectCreate(obj, OBJ_LABEL, winID ,0,0)) ObjectCreate(obj, OBJ_LABEL, winID ,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))      ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))      ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, 0))         ObjectSet(obj, OBJPROP_CORNER, 0);
   if(!ObjectSetText(obj, data, fontSize, font, clrText))  ObjectSetText(obj, data, fontSize, font, clrText);

   d++;
   x += 100;
   if(d % 10 == 0) {x = 22; y += 22;}
   if(d % 40 == 0) {d = 0; x = 22; y = 22;}

   return;
   } // end void Labels(string data)

//+------------------------------------------------------------------+

void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+--------------------------------EOP-------------------------------+
