//+------------------------------------------------------------------+
//|   DEMO 2                            00z Reverse Z Functions.mq4  |
//|   Last edit: August 8, 2014                                      |
//|   3 functions that return original Close, High and Low Prices.   |
//|   double BarClose(double zClose, int period, int bar),           |
//|   BarHigh(..) and BarLow(..)                                     |
//|   Each function can reference any TF and Bar, by passing         |
//|   a Z-score, specific Period() and Bar Index.                    |
//|                                                                  |
//|   The function GetScores(int period, int bar) generates z-scores.|
//|   In this DEMO we ONLY call the current Bars #0 on Every Tick.   |
//+------------------------------------------------------------------+
#property copyright " 2011-2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window

extern string  Make_Sure = "   Set SMA Period equal to BB system!";
extern string SMA_Period = "   Default Simple MA Period = 20";
extern int     smaPeriod = 20; // 20
extern string       font = "Arial";
extern string   fontSize = 10; // 10
extern color     clrText = clrGray;       //default only, user color choice
extern color    clrPrice = clrDodgerBlue; //clrDodgerBlue; // last 3 columns

string tf[9] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int   per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
      PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

double zScore, zHi, zLo; // global z-values
string pref = "z2p_";
color clr;
int winID; // subwindow

//+------------------------------------------------------------------+

int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;
   clr = clrText;

   IndicatorShortName("3_Reverse_Z_Functions");
   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
  {
   Clear(pref, 0);
   return(0);
  } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(winID < 1) winID = WindowFind("3_Reverse_Z_Functions");
   // labels run left to right, top to bottom, setting headers first
   clr = clrText;
   Labels(StringConcatenate(smaPeriod, " SMA"));
   Labels("Z Score");
   Labels("Z High");
   Labels("Z Low");

   clr = Crimson;
   Labels(DoubleToStr(Close[0], Digits));

   clr = clrPrice;
   Labels("Bar High");
   Labels("Bar Low");

   for(int i=8; i>=0; i--)
      {
      clr = clrText;
      // get all 3 z-values in global scope by calling GetScores(..)
      GetScores(per[i], 0);

      Labels(tf[i]);
      Labels(DoubleToStr(zScore, 8));
      Labels(DoubleToStr(zHi, 8));
      Labels(DoubleToStr(zLo, 8));

      clr = clrPrice;
      // return 3 Price Values by making individual calls, pass the z-values
      Labels(DoubleToStr(BarClose(zScore, per[i], 0), Digits));
      Labels(DoubleToStr(BarHigh(zHi, per[i], 0), Digits));
      Labels(DoubleToStr(BarLow(zLo, per[i], 0), Digits));
      }

   return(0);
   } // end int start()

//+--------------------------GET 3 Z FUNCTION------------------------+

void GetScores(int period, int bar)
   { // manage z variables naming here
     // calculates all 3 z-vals in one pass, uses globally declared variables
   double SMA =     iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar);
   double SD  = iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar);

   zScore = (iClose(NULL, period, bar) - SMA) / SD;
   zHi    = ( iHigh(NULL, period, bar) - SMA) / SD;
   zLo    = (  iLow(NULL, period, bar) - SMA) / SD;

   return;
   } // end double GetScores()

//+--------------------------REVERSE FUNCTIONS-----------------------+
/*
   from z-scores to originally observed price:
   price value = (z-score * SD) + SMA
*/

double BarClose(double zClose, int period, int bar)
   {
  return((zClose * iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))+
                       iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetBarClose(int period, int bar)

//+------------------------------------------------------------------+

double BarHigh(double zHigh, int period, int bar)
   {
   return((zHigh * iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))+
                       iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetBarHi(int period, int bar)

//+------------------------------------------------------------------+

double BarLow(double zLow, int period, int bar)
   {
   return((zLow * iStdDev(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar))+
                      iMA(NULL, period, smaPeriod, 0,MODE_SMA,PRICE_CLOSE, bar));
   } // end double GetBarLo(int period, int bar)

//+---------------------------END FUNCTIONS--------------------------+

void Labels(string data)
   {
   static int d = 0; // label ID
   static int x = 22;
   static int y = 22;

   string obj = StringConcatenate(pref, d);
   if(!ObjectCreate(obj, OBJ_LABEL, winID ,0,0))  ObjectCreate(obj, OBJ_LABEL, winID ,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))      ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))      ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, 0))         ObjectSet(obj, OBJPROP_CORNER, 0);
   if(!ObjectSetText(obj, data, fontSize, font, clr))  ObjectSetText(obj, data, fontSize, font, clr);

   d++;
   x += 100;

   if(d % 7 == 0) {x = 22; y += 22;}
   if(d % 70 == 0) {d = 0; x = 22; y = 22;}

   return;
   } // end void Labels(string data)

//+------------------------------------------------------------------+

void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+--------------------------------EOP-------------------------------+
